import subprocess
import json
import tempfile
import os
from .utils import clean_spotify_url

def fetch_info(*, URL):
    url = clean_spotify_url(URL)
    
    # Crée un fichier temporaire
    with tempfile.NamedTemporaryFile(mode='w', suffix='.spotdl', delete=False) as tmp:
        tmp_path = tmp.name
    
    try:
        # Sauvegarde les métadonnées
        result = subprocess.run(
            ["spotdl", "save", url, "--save-file", tmp_path], 
            capture_output=True, 
            text=True
        )
        
        if result.returncode == 0 and os.path.exists(tmp_path):
            # Lit le fichier JSON
            with open(tmp_path, 'r', encoding='utf-8') as f:
                data = json.load(f)
            return data # Renvoie des donnée qui nous interesse
        else:
            print("Erreur:", result.stderr)
            return None
    finally:
        # Nettoie immédiatement le fichier tmp
        if os.path.exists(tmp_path):
            os.remove(tmp_path)


import os

class Utils:
    def get_ydl_opts(output_path=None, quality="bv*+ba/b", cookies_path=None, logger=None):
        opts = {
            "format": quality,
            "ignoreerrors": True,
            "quiet": True,
            "merge_output_format": "mp4",
        }

        if output_path:
            opts["outtmpl"] = os.path.join(output_path, "%(title)s.%(ext)s")

        # Cookies dans le cas ou l'utilisateur serait un premium pour télécharger la video en 1080p par exemple
        if cookies_path:
            opts["cookies"] = cookies_path

        if logger:
            opts["logger"] = logger
            opts["progress_hooks"] = [logger.hook]

        return opts



import os, yt_dlp
from .utils import YuiCleanLogger, Kit

def _download_generic(url, folder_name, *, quality="best", PATH=None):
    """Fonction centrale pour tout téléchargement."""
    try : 
        if not PATH:
            AP = os.getcwd()
            PATH = os.path.join(AP, f"{folder_name}")

        cleanLogger = YuiCleanLogger()

        ydl_opts = {
            'outtmpl': os.path.join(PATH, '%(title)s.%(ext)s'),         # Ouput directement envoie avec le titre de la video dans le bon dossier
            'format': quality,                                          # Format flexible
            'ignoreerrors': True,                                       # Ignorer certaines erreurs non critiques
            "logger": cleanLogger,                                      # Logger spécifique pour retirer tous se qui est de trop
            "progress_hooks": [cleanLogger.hook],                       # Pour un affichage personnalisé de la progression
            'verbose': False                                            # Afficher moins d'informationsyt-dl
            }

        with yt_dlp.YoutubeDL(ydl_opts) as ydl:
            return ydl.download([url])
        
    except KeyboardInterrupt:
        return None
    
    
class Info:
    def _fetch_info(*, url):
        with yt_dlp.YoutubeDL(Kit.OPTS) as yt:
            return yt.extract_info(url, download=False)
        
    def _get_title(info):
        """
        Returns the title of the YouTube video.

        Can be None if the information is missing or extraction failed.
        """
        return info.get("title")

    def _get_creator(info):
        """
        Returns the name of the creator or uploader of the video.

        yt-dlp may provide this information via different fields depending on the video.
        """
        return info.get("creator")

    def _get_like_count(info):
        """
        Returns the number of likes for the video.

        Can be None if the data is unavailable.
        """
        return info.get("like_count")

    def _get_age_limit(info):
        """
        Returns the age restriction for the video (e.g., 18).

        None if no restriction is set.
        """
        return info.get("age_limit")

    def _get_availability(info):
        """
        Returns the availability status of the video.

        Examples: "public", "private", "unlisted", etc.
        """
        return info.get("availability")

    def _get_available_at(info):
        """
        Returns the Unix timestamp (in seconds) when the video becomes available.

        None if not defined.
        """
        return info.get("available_at")

    def _get_comment_count(info):
        """
        Returns the number of comments on the video.

        Can be None if comments are disabled or unavailable.
        """
        return info.get("comment_count")

    def _get_duration(info):
        """
        Returns the duration of the video in seconds.

        Can be None if the duration is unknown.
        """
        return info.get("duration")

    def _get_formats(info):
        """
        Returns the list of available formats for the video.

        Each format is a dictionary provided by yt-dlp.
        """
        return info.get("formats")

    def _get_id(info):
        """
        Returns the unique ID of the YouTube video.
        """
        return info.get("id")

    def _get_tags(info):
        """
        Returns the tags associated with the video.

        Can be an empty list or None if no tags are defined.
        """
        return info.get("tags")

    def _get_thumbnail(info):
        """
        Returns the URL of the main thumbnail of the video.
        """
        return info.get("thumbnail")

    def _get_timestamp(info):
        """
        Returns the Unix timestamp of the video publication.

        None if unknown.
        """
        return info.get("timestamp")

    def _get_url(info):
        """
        Returns the URL of the YouTube video.
        """
        return info.get("url")


import os
import yt_dlp
from ..core.utils import YuiCleanLogger
from .utils import Utils

def download(url, *, quality="bv*+ba/b", PATH=None, cookies_path=None):
    """Téléchargement Bilibili"""

    try:
        if not PATH:
            AP = os.getcwd()
            PATH = os.path.join(AP, "Bilibili_Download")

        os.makedirs(PATH, exist_ok=True)

        cleanLogger = YuiCleanLogger()

        # ydl_opts = {
        #     "outtmpl": os.path.join(PATH, "%(title)s.%(ext)s"),             # Dossier et nom fichier            
        #     "format": quality,                                              # bv* = meilleure vidéo / ba = meilleur audio
        #     "merge_output_format": "mp4",                                   # Passe la video en mp4 automatiquement
        #     "ignoreerrors": True,                                           # Evite les erreur non critique pour continuer malgré tous
        #     "quiet": True,                                                  # Evite la saturation du terminal
        #     "verbose": False,                                               # Evite les log inutile
        #     "logger": cleanLogger,                                          # Logger sépcifique
        #     "progress_hooks": [cleanLogger.hook],                           # Progresse bar spécifique
        # }

        ydl_opts = Utils.get_ydl_opts(
            output_path=PATH,
            quality=quality,
            cookies_path=cookies_path,
            logger=cleanLogger
        )


        with yt_dlp.YoutubeDL(ydl_opts) as ydl:
            return ydl.download([url])

    except KeyboardInterrupt:
        return None



import os, subprocess

def download(URL, *, PATH=None):
    """Téléchargement Spotify"""
    try:
        if not PATH:
            AP = os.getcwd()
            PATH = os.path.join(AP, "Spotify_Download")
    
        subprocess.run(["spotdl", URL, "--output", PATH], capture_output=True, text=True)

    except KeyboardInterrupt:
        return None


def get_title(info):
    """
    Return the titles of all music tracks.
    
    :param info: List of song dictionaries
    :return: List of titles
    """
    title = []
    for song in info:
        title.append({song['name']})
    return title

def get_creator(info):
    """
    Return the unique creators of all music
    
    :param info: List of song dictionaries
    :return: List of unique artists
    """
    artists = set()
    for song in info:
        artists.update(song['artists'])
    return list(artists)



from .downloader import download
from .fetch import fetch_info
from .info import (
    get_title,
    get_like_count,
    get_creator,
    get_age_limit,
    get_availability,
    get_available_at,
    get_comment_count,
    get_duration,
    get_formats,
    get_id,
    get_tags,
    get_thumbnail,
    get_timestamp,
    get_url,
)


from ..core.core import Info

def get_title(info):
    """
    Returns the title of the YouTube video.

    Can be None if the information is missing or extraction failed.
    """
    return Info._get_title(info)

def get_creator(info):
    """
    Returns the name of the creator or uploader of the video.

    yt-dlp may provide this information via different fields depending on the video.
    """
    return Info._get_creator(info)

def get_like_count(info):
    """
    Returns the number of likes for the video.

    Can be None if the data is unavailable.
    """
    return Info._get_like_count(info)

def get_age_limit(info):
    """
    Returns the age restriction for the video (e.g., 18).

    None if no restriction is set.
    """
    return Info._get_age_limit(info)

def get_availability(info):
    """
    Returns the availability status of the video.

    Examples: "public", "private", "unlisted", etc.
    """
    return Info._get_availability(info)

def get_available_at(info):
    """
    Returns the Unix timestamp (in seconds) when the video becomes available.

    None if not defined.
    """
    return Info._get_available_at(info)

def get_comment_count(info):
    """
    Returns the number of comments on the video.

    Can be None if comments are disabled or unavailable.
    """
    return Info._get_comment_count(info)

def get_duration(info):
    """
    Returns the duration of the video in seconds.

    Can be None if the duration is unknown.
    """
    return Info._get_duration(info)

def get_formats(info):
    """
    Returns the list of available formats for the video.

    Each format is a dictionary provided by yt-dlp.
    """
    return Info._get_formats(info)

def get_id(info):
    """
    Returns the unique ID of the YouTube video.
    """
    return Info._get_id(info)

def get_tags(info):
    """
    Returns the tags associated with the video.

    Can be an empty list or None if no tags are defined.
    """
    return Info._get_tags(info)

def get_thumbnail(info):
    """
    Returns the URL of the main thumbnail of the video.
    """
    return Info._get_thumbnail(info)

def get_timestamp(info):
    """
    Returns the Unix timestamp of the video publication.

    None if unknown.
    """
    return Info._get_timestamp(info)

def get_url(info):
    """
    Returns the URL of the YouTube video.
    """
    return Info._get_url(info)


import re

def clean_spotify_url(url: str) -> str:
    url = url.strip()

    # spotify:track:ID
    m = re.match(r"spotify:(track|playlist|album):([A-Za-z0-9]+)", url)
    if m:
        kind, sid = m.groups()
        return f"https://open.spotify.com/{kind}/{sid}"

    # https://open.spotify.com/.../type/ID?... 
    m = re.search(
        r"open\.spotify\.com/(?:intl-[a-z]{2}/)?(track|playlist|album)/([A-Za-z0-9]+)",
        url
    )
    if m:
        kind, sid = m.groups()
        return f"https://open.spotify.com/{kind}/{sid}"

    raise ValueError(f"URL Spotify invalide: {url}")



import shutil, sys

class YuiCleanLogger:
    def __init__(self):
        self.path_printed = False

    def debug(self, msg):
        # print(msg) # A décommentez pour le débug
        pass # Ignore les messages de type '[info]', '[generic]', etc.

    def warning(self, msg):
        # print(msg) # A  décommentez pour voir les avertissements
        pass # Ignore les avertissements

    def error(self, msg):
        print(msg)

    def hook(self, d):
        if d['status'] == 'downloading':
            if not self.path_printed:
                print(f"Destination: {d['filename']}")
                self.path_printed = True

            # La ligne de progression qui se met à jour
            progress_line = (
                f"[download] {d['_percent_str']} of {d.get('total_bytes_str', 'N/A')}"
                f" at {d['_speed_str']} ETA {d['_eta_str']}"
            )
            
            try:
                # Récupère la largeur actuelle du terminal
                terminal_width = shutil.get_terminal_size().columns
            except OSError:
                 # Largeur par défaut pour éviter les crashes    
                terminal_width = 80
            
            line_to_write = f"\r{progress_line:<{terminal_width - 1}}"
            sys.stdout.write(line_to_write) 
            sys.stdout.flush()

        if d['status'] == 'finished':
            # print(self.languages[self.langue]["ytDlpFinish"])
            # Réinitialise la variable pour le prochain fichier à télécharger
            self.path_printed = False

class Kit:
    cleanLogger = YuiCleanLogger()

    # OPTS utiliser uniquement pour le fetch des info youtube et tiktok
    OPTS = {
        'format': "best",                                               # Format plus flexible
        'ignoreerrors': True,                                           # Ignorer certaines erreurs non critiques
        "logger": cleanLogger,                                          # Logger spécifique pour retirer tous se qui est de trop
        "progress_hooks": [cleanLogger.hook],                           # Pour un affichage personnalisé de la progression
        'verbose': False                                                # Afficher plus d'informations
    }